@@ -33,7 +33,6 @@ Black::Black(Real x, Real y) : Ball() {
   position[0] = x;
   position[1] = y;
   position[2] = Game::current->map->getHeight(position[0], position[1]) + radius;
-  zero(velocity);
   crashTolerance = 7;
 
   /* Set color to black */
@@ -45,7 +44,6 @@ Black::Black(Real x, Real y) : Ball() {
   specularColor[2] = 1.0;
 
   bounceFactor = .8;
-  acceleration = 4.0;
   horizon = 5.0;
   likesPlayer = 1;
 
@@ -88,31 +86,34 @@ void Black::tick(Real t) {
   double dist;
 
   if (Game::current->player1->playing && is_on) {
-    sub(Game::current->player1->position, position, v);
+    Coord3d v = Game::current->player1->position - position;
     dist = length(v);
 
     double d = Game::current->map->getHeight(position[0] + velocity[0] * 1.0,
                                              position[1] + velocity[1] * 1.0);
-    Cell& c1 = Game::current->map->cell((int)position[0], (int)position[1]);
     Cell& c2 = Game::current->map->cell((int)(position[0] + velocity[0] * 1.0),
                                         (int)(position[1] + velocity[1] * 1.0));
 
+    bool expect_fall = d < position[2] - 1.0 && !modTimeLeft[MOD_FLOAT];
+    bool death_cell = c2.flags & CELL_ACID || c2.flags & CELL_KILL;
+
     /* TODO. Make these checks better */
-    if ((d < position[2] - 1.0 && !modTimeLeft[MOD_FLOAT]) ||
-        (!(c1.flags & CELL_ACID) && (c2.flags & CELL_ACID))) {
-      /* Stop, we are near an edge  or acid or ... */
+    if ((expect_fall || death_cell) && !inPipe) {
+      /* Stop, we are near an edge or acid or ... */
       v[0] = velocity[0];
       v[1] = velocity[1];
       v[2] = 0.0;
-      normalize(v);
-      rotation[0] -= v[0] * acceleration * t;
-      rotation[1] -= v[1] * acceleration * t;
+      double vsc = length(v) > 0 ? 1. / length(v) : 0.;
+      v = v * vsc;
+      Ball::drive(-v[0], -v[1]);
     } else if (dist < horizon) {
       /* Go toward the player */
-      normalize(v);
-      rotation[0] += v[0] * acceleration * likesPlayer * t;
-      rotation[1] += v[1] * acceleration * likesPlayer * t;
+      double vsc = length(v) > 0 ? 1. / length(v) : 0.;
+      v = v * vsc;
+      Ball::drive(v[0] * likesPlayer, v[1] * likesPlayer);
     }
+  } else {
+    Ball::drive(0., 0.);
   }
 
   Ball::tick(t);
