@@ -159,6 +159,8 @@ int doAskSave = 0;
 
 int selectedMenu = -1;
 
+const int viewPoints[4][2] = {{1, 1}, {1, -1}, {-1, -1}, {-1, 1}};
+
 EditMode* EditMode::editMode;
 
 void EditMode::loadStrings() {
@@ -367,7 +369,7 @@ void EditMode::display() {
   activeView.fog_enabled = 0;
   activeView.quadratic_attenuation = 0.;
 
-  Coord3d lightPosition = {x - 60., y - 40., h + 100.};
+  GLfloat lightPosition[3] = {(GLfloat)x - 60.f, (GLfloat)y - 40.f, (GLfloat)h + 100.f};
   GLfloat lightDiffuse[3] = {0.9, 0.9, 0.9};
   GLfloat ambient[3] = {0.2, 0.2, 0.2};
   GLfloat black[3] = {0., 0., 0.};
@@ -380,20 +382,17 @@ void EditMode::display() {
   /* Setup matrixes for the camera perspective */
   perspectiveMatrix(40, (GLdouble)screenWidth / (GLdouble)std::max(screenHeight, 1), 0.1, 200,
                     activeView.projection);
-  if (!switchViewpoint) {
-    lookAtMatrix(x - 7.0, y - 7.0, (birdsEye ? 30.0 : 10.0) + h * 0.5, x, y, h, 0.0, 0.0, 1.0,
-                 activeView.modelview);
-  } else {
-    lookAtMatrix(x + 7.0, y + 7.0, (birdsEye ? 30.0 : 10.0) + h * 0.5, x, y, h, 0.0, 0.0, 1.0,
-                 activeView.modelview);
-  }
+
+  lookAtMatrix(x - viewPoints[switchViewpoint][0] * 7.0,
+               y - viewPoints[switchViewpoint][1] * 7.0, (birdsEye ? 30.0 : 10.0) + h * 0.5, x,
+               y, h, 0.0, 0.0, 1.0, activeView.modelview);
 
   activeView.day_mode = 1;
   updateUniforms();
   /* Shadow map rendering returns active modelview/projection to orig state */
   if (map) {
     if (Settings::settings->doShadows) {
-      Coord3d focus = {(double)x, (double)y, map->getHeight(x, y)};
+      Coord3d focus((double)x, (double)y, map->getHeight(x, y));
       renderShadowCascade(focus, map, game);
       renderDummyShadowMap();
     } else {
@@ -786,16 +785,20 @@ void EditMode::doCommand(int command) {
   } break;
 
   case MOVE_UP:
-    x += (switchViewpoint ? -1 : 1) * (ctrl ? 20 : (shift ? 5 : 1));
+    ((switchViewpoint % 2) ? y : x) +=
+        viewPoints[switchViewpoint][1] * (ctrl ? 20 : (shift ? 5 : 1));
     break;
   case MOVE_DOWN:
-    x -= (switchViewpoint ? -1 : 1) * (ctrl ? 20 : (shift ? 5 : 1));
+    ((switchViewpoint % 2) ? y : x) -=
+        viewPoints[switchViewpoint][1] * (ctrl ? 20 : (shift ? 5 : 1));
     break;
   case MOVE_LEFT:
-    y += (switchViewpoint ? -1 : 1) * (ctrl ? 20 : (shift ? 5 : 1));
+    ((switchViewpoint % 2) ? x : y) +=
+        viewPoints[switchViewpoint][0] * (ctrl ? 20 : (shift ? 5 : 1));
     break;
   case MOVE_RIGHT:
-    y -= (switchViewpoint ? -1 : 1) * (ctrl ? 20 : (shift ? 5 : 1));
+    ((switchViewpoint % 2) ? x : y) -=
+        viewPoints[switchViewpoint][0] * (ctrl ? 20 : (shift ? 5 : 1));
     break;
 
   case MOVE_SET_MARKER:
@@ -825,7 +828,7 @@ void EditMode::doCommand(int command) {
     birdsEye = birdsEye ? 0 : 1;
     break;
   case VIEW_ROTATE:
-    switchViewpoint = switchViewpoint ? 0 : 1;
+    switchViewpoint = (switchViewpoint + 1) % 4;
     break;
   case VIEW_CLEAR_ENTITIES:
     if (game) delete game;
@@ -1048,13 +1051,13 @@ void EditMode::doCellAction(int code, int direction) {
 }
 
 void EditMode::key(int key) {
-  int shift = SDL_GetModState() & (KMOD_LSHIFT | KMOD_RSHIFT);
+  int shift = SDL_GetModState() & KMOD_SHIFT;
   int mouseX, mouseY;
   SDL_GetMouseState(&mouseX, &mouseY);
   /* TODO. Send any keys to the EOpenWindow?? */
 
   if (newWindow->isAttached()) {
-    newWindow->key(key, mouseX, mouseY);
+    newWindow->key(key, shift, mouseX, mouseY);
     return;
   }
 
@@ -1105,20 +1108,11 @@ void EditMode::key(int key) {
     return;
   }
 
-  /* Move cursor */
-  /*
-  switch(key) {
-  case SDLK_LEFT: y+=(switchViewpoint?-1:1)*(shift?5:1); break;
-  case SDLK_RIGHT: y-=(switchViewpoint?-1:1)*(shift?5:1); break;
-  case SDLK_UP: x+=(switchViewpoint?-1:1)*(shift?5:1); break;
-  case SDLK_DOWN: x-=(switchViewpoint?-1:1)*(shift?5:1); break;
-  }*/
-
   if (key == SDLK_TAB) { moveKeyboardFocus(shift); }
   if (key == SDLK_RETURN || key == SDLK_KP_ENTER || key == SDLK_SPACE)
     mouseDown(shift ? 3 : 1, -1, -1);
 
-  menuWindow->key(key, shift);
+  menuWindow->key(key, shift, x, y);
 
   return;
 }
