@@ -30,10 +30,9 @@
 
 #include <SDL2/SDL_image.h>
 #include <SDL2/SDL_ttf.h>
+#include <cstdlib>
 #include <map>
 
-#define SHADOW_TEXSIZE 512
-
 float fps = 50.0;
 int screenWidth = 640, screenHeight = 480;
 ViewParameters activeView;
@@ -49,6 +48,7 @@ GLuint theVao = 0;
 TTF_Font *ingameFont;
 extern struct timespec displayStartTime;
 extern struct timespec lastDisplayStartTime;
+extern double timeDilationFactor;
 
 const GLfloat menuColorSelected[4] = {0.86f, 0.86f, 0.86f, 1.f};
 const GLfloat menuColor[4] = {0.86f, 0.86f, 0.25f, 1.f};
@@ -73,10 +73,6 @@ ushort *sphere_idxs[MAX_BALL_DETAIL];
 static std::map<int, TTF_Font *> menuFontLookup;
 
 Sparkle2D *sparkle2D = NULL;
-/*
-SDL_Surface *mousePointer;
-GLuint mousePointerTexture;
-*/
 
 struct StringInfo {
   TTF_Font *font;
@@ -221,9 +217,9 @@ void tickMouse(Real td) {
 void draw2DRectangle(GLfloat x, GLfloat y, GLfloat w, GLfloat h, GLfloat tx, GLfloat ty,
                      GLfloat tw, GLfloat th, GLfloat r, GLfloat g, GLfloat b, GLfloat a,
                      GLuint tex) {
-  GLfloat corners[4][2] = {x, y, x, y + h, x + w, y, x + w, y + h};
-  GLfloat texture[4][2] = {tx, ty, tx, ty + th, tx + tw, ty, tx + tw, ty + th};
-  GLfloat colors[4][4] = {r, g, b, a, r, g, b, a, r, g, b, a, r, g, b, a};
+  GLfloat corners[4][2] = {{x, y}, {x, y + h}, {x + w, y}, {x + w, y + h}};
+  GLfloat texture[4][2] = {{tx, ty}, {tx, ty + th}, {tx + tw, ty}, {tx + tw, ty + th}};
+  GLfloat colors[4][4] = {{r, g, b, a}, {r, g, b, a}, {r, g, b, a}, {r, g, b, a}};
   draw2DQuad(corners, texture, colors, tex);
 }
 
@@ -281,7 +277,8 @@ void drawMouse(int x, int y, int w, int h) {
   GLfloat dx = 0.707f * w * r1 * std::sin(mousePointerPhase * 0.35);
   GLfloat dy = 0.707f * h * r2 * std::cos(mousePointerPhase * 0.35);
 
-  GLfloat colors[4][4] = {1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1.};
+  GLfloat colors[4][4] = {
+      {1., 1., 1., 1.}, {1., 1., 1., 1.}, {1., 1., 1., 1.}, {1., 1., 1., 1.}};
   GLfloat texco[4][2] = {{0., 0.}, {0., 1.}, {1., 0.}, {1., 1.}};
 
   GLfloat vco[4][2] = {{x - dx, y - dy}, {x - dy, y + dx}, {x + dy, y - dx}, {x + dx, y + dy}};
@@ -491,16 +488,15 @@ void perspectiveMatrix(GLdouble fovy_deg, GLdouble aspect, GLdouble zNear, GLdou
 void lookAtMatrix(GLdouble eyeX, GLdouble eyeY, GLdouble eyeZ, GLdouble centerX,
                   GLdouble centerY, GLdouble centerZ, GLdouble upX, GLdouble upY, GLdouble upZ,
                   Matrix4d out) {
-  Coord3d eye = {eyeX, eyeY, eyeZ};
-  Coord3d center = {centerX, centerY, centerZ};
-  Coord3d up = {upX, upY, upZ};
-  Coord3d f, s, u;
-  sub(center, eye, f);
-  normalize(f);
-  normalize(up);
-  crossProduct(f, up, s);
-  normalize(s);
-  crossProduct(s, f, u);
+  Coord3d eye(eyeX, eyeY, eyeZ);
+  Coord3d center(centerX, centerY, centerZ);
+  Coord3d up(upX, upY, upZ);
+  Coord3d f = center - eye;
+  f = f / length(f);
+  up = up / length(up);
+  Coord3d s = crossProduct(f, up);
+  s = s / length(s);
+  Coord3d u = crossProduct(s, f);
 
   Matrix4d mat = {{s[0], u[0], -f[0], 0.},
                   {s[1], u[1], -f[1], 0.},
@@ -530,9 +526,8 @@ void setViewUniforms(GLuint shader) {
   glUniform1f(glGetUniformLocation(shader, "fog_start"), activeView.fog_start);
   glUniform1f(glGetUniformLocation(shader, "fog_end"), activeView.fog_end);
 
-  glUniform3f(glGetUniformLocation(shader, "light_position"),
-              (GLfloat)activeView.light_position[0], (GLfloat)activeView.light_position[1],
-              (GLfloat)activeView.light_position[2]);
+  glUniform3f(glGetUniformLocation(shader, "light_position"), activeView.light_position[0],
+              activeView.light_position[1], activeView.light_position[2]);
   glUniform3f(glGetUniformLocation(shader, "light_ambient"), activeView.light_ambient[0],
               activeView.light_ambient[1], activeView.light_ambient[2]);
   glUniform3f(glGetUniformLocation(shader, "light_diffuse"), activeView.light_diffuse[0],
@@ -586,7 +581,7 @@ void setViewUniforms(GLuint shader) {
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 
-  int N = 3;
+  const int N = 3;
   GLfloat cscproj[N * 16], cscmodel[N * 16];
   for (int i = 0; i < N; i++) {
     for (int j = 0; j < 4; j++) {
@@ -634,7 +629,9 @@ void renderShadowMap(Coord3d focus, Map *mp, Game *gm) {
   assign(activeView.projection, origProj);
 
   activeView.calculating_shadows = 1;
-  if (activeView.shadowMapTexsize <= 1) {
+  static int ltexsize = 0;
+  if (activeView.shadowMapTexsize <= 1 || ltexsize != Settings::settings->shadowTexsize) {
+    ltexsize = Settings::settings->shadowTexsize;
     /* order doesn't matter */
     GLenum dirs[6] = {GL_TEXTURE_CUBE_MAP_POSITIVE_X, GL_TEXTURE_CUBE_MAP_NEGATIVE_X,
                       GL_TEXTURE_CUBE_MAP_POSITIVE_Y, GL_TEXTURE_CUBE_MAP_NEGATIVE_Y,
@@ -642,15 +639,16 @@ void renderShadowMap(Coord3d focus, Map *mp, Game *gm) {
 
     GLint maxSize;
     glGetIntegerv(GL_MAX_TEXTURE_SIZE, &maxSize);
-    activeView.shadowMapTexsize = std::min(maxSize, SHADOW_TEXSIZE);
+    GLint reqSize = 1 << Settings::settings->shadowTexsize;
+    activeView.shadowMapTexsize = std::min(maxSize, reqSize);
     for (uint face = 0; face < 6; face++) {
       glTexImage2D(dirs[face], 0, GL_DEPTH_COMPONENT, activeView.shadowMapTexsize,
                    activeView.shadowMapTexsize, 0, GL_DEPTH_COMPONENT, GL_FLOAT, NULL);
     }
   }
 
-  Coord3d norv[6] = {{1, 0, 0}, {-1, 0, 0}, {0, 1, 0}, {0, -1, 0}, {0, 0, 1}, {0, 0, -1}};
-  Coord3d upv[6] = {{0, -1, 0}, {0, -1, 0}, {0, 0, 1}, {0, 0, -1}, {0, -1, 0}, {0, -1, 0}};
+  GLdouble norv[6][3] = {{1, 0, 0}, {-1, 0, 0}, {0, 1, 0}, {0, -1, 0}, {0, 0, 1}, {0, 0, -1}};
+  GLdouble upv[6][3] = {{0, -1, 0}, {0, -1, 0}, {0, 0, 1}, {0, 0, -1}, {0, -1, 0}, {0, -1, 0}};
   /* order doesn't matter */
   GLenum dirs[6] = {GL_TEXTURE_CUBE_MAP_POSITIVE_X, GL_TEXTURE_CUBE_MAP_NEGATIVE_X,
                     GL_TEXTURE_CUBE_MAP_POSITIVE_Y, GL_TEXTURE_CUBE_MAP_NEGATIVE_Y,
@@ -707,7 +705,7 @@ void mulMatrix4(const Matrix4d mult, const double src[4], double res[4]) {
   for (int i = 0; i < 4; i++) {
     for (int j = 0; j < 4; j++) { r[i] += src[j] * mult[i][j]; }
   }
-  assign(r, res);
+  for (int i = 0; i < 4; i++) { res[i] = r[i]; }
 }
 
 void renderShadowCascade(Coord3d focus, Map *mp, Game *gm) {
@@ -734,10 +732,8 @@ void renderShadowCascade(Coord3d focus, Map *mp, Game *gm) {
   Matrix3d mvm3t;
   for (int k = 0; k < 3; k++)
     for (int j = 0; j < 3; j++) mvm3t[j][k] = mvmt[k][j];
-  Coord3d cc = {mvmt[0][3], mvmt[1][3], mvmt[2][3]};
-  Coord3d camera;
-  useMatrix(mvm3t, cc, camera);
-  for (int k = 0; k < 3; k++) camera[k] *= -1;
+  Coord3d cc(mvmt[0][3], mvmt[1][3], mvmt[2][3]);
+  Coord3d camera = -useMatrix(mvm3t, cc);
 
   Matrix4d light_align;
   lookAtMatrix(0, 0, 0, activeView.sun_direction[0], activeView.sun_direction[1],
@@ -755,15 +751,12 @@ void renderShadowCascade(Coord3d focus, Map *mp, Game *gm) {
                           aspect * std::tan(proj_half_angle) * dts[i + j], dts[i + j]};
       for (int k = 0; k < 4; k++) {
         eyepts[k % 2] *= -1;
-        double resultA[3];
         /* convert to world space */
-        useMatrix(mvm3t, eyepts, resultA);
-        for (int m = 0; m < 3; m++) resultA[m] += camera[m];
+        Coord3d resultA = useMatrix(mvm3t, eyepts) + camera;
 
-        double result[3];
         /* reorient along light space. NOTE: this breaks total offset, but we don't care about
          * it...*/
-        useMatrix(light_align3, resultA, result);
+        Coord3d result = useMatrix(light_align3, resultA);
 
         for (int m = 0; m < 3; m++) {
           bounds[m][0] = std::min(bounds[m][0], result[m]);
@@ -774,15 +767,13 @@ void renderShadowCascade(Coord3d focus, Map *mp, Game *gm) {
     double dx = bounds[0][1] - bounds[0][0], dy = bounds[1][1] - bounds[1][0];  //,
     double rx = std::max(dx, dy);
     double ry = std::max(dx, dy);
-    double midpoint[3] = {0, 0, -0.5 * (dts[i] + dts[i + 1])};
-    double mmpt[3];
-    useMatrix(mvm3t, midpoint, mmpt);
-    for (int k = 0; k < 3; k++) mmpt[k] += camera[k];
+    Coord3d midpoint(0, 0, -0.5 * (dts[i] + dts[i + 1]));
+    Coord3d mmpt = useMatrix(mvm3t, midpoint) + camera;
 
     double s = 50; /* rel. the 100 of range */
-    Coord3d light_camera = {mmpt[0] - s * activeView.sun_direction[0],
-                            mmpt[1] - s * activeView.sun_direction[1],
-                            mmpt[2] - s * activeView.sun_direction[2]};
+    Coord3d sundir(activeView.sun_direction[0], activeView.sun_direction[1],
+                   activeView.sun_direction[2]);
+    Coord3d light_camera = mmpt - s * sundir;
     lookAtMatrix(light_camera[0], light_camera[1], light_camera[2], mmpt[0], mmpt[1], mmpt[2],
                  0, 0, 1, activeView.cascade_model[i]);
 
@@ -794,11 +785,14 @@ void renderShadowCascade(Coord3d focus, Map *mp, Game *gm) {
   }
 
   /* Render cascade map using the given matrices */
-  if (activeView.cascadeTexsize <= 1) {
+  static int ltexsize = 0;
+  if (activeView.cascadeTexsize <= 1 || ltexsize != Settings::settings->shadowTexsize) {
+    ltexsize = Settings::settings->shadowTexsize;
     /* order doesn't matter */
     GLint maxSize;
     glGetIntegerv(GL_MAX_TEXTURE_SIZE, &maxSize);
-    activeView.cascadeTexsize = std::min(maxSize, SHADOW_TEXSIZE);
+    GLint reqSize = 1 << Settings::settings->shadowTexsize;
+    activeView.cascadeTexsize = std::min(maxSize, reqSize);
   }
   for (int i = 0; i < N; i++) {
     glBindTexture(GL_TEXTURE_2D, activeView.cascadeTexture[i]);
@@ -850,7 +844,7 @@ void renderShadowCascade(Coord3d focus, Map *mp, Game *gm) {
 int createSnapshot() {
   static int snap_number = 0;
   char name[1024];
-  int again = 9999, i, j;
+  int again = 9999;
   FILE *f;
 
   /* find the name for the image */
@@ -1225,7 +1219,6 @@ int resetTextures() {
   GLfloat texCoord[4];
   char str[256];
   SDL_Surface *surface;
-  int linear = 0;
 
   for (int i = 0; i < numTextures; i++) {
     if (textureNames[i] == NULL) continue;
@@ -1246,7 +1239,7 @@ int resetTextures() {
 /* Calculates and displays current framerate */
 void displayFrameRate() {
   /* Frame rate is given for the previous period. */
-  double td = getTimeDifference(lastDisplayStartTime, displayStartTime);
+  double td = getTimeDifference(lastDisplayStartTime, displayStartTime) / timeDilationFactor;
   if (td > 1.0)
     fps = 1.0;
   else if (td <= 1e-4) {
@@ -1293,7 +1286,8 @@ void matrixMult(const Matrix4d A, const Matrix4d B, Matrix4d C) {
 }
 
 /* C <- A(B) */
-void useMatrix(Matrix4d A, const double B[3], double C[3]) {
+Coord3d useMatrix(Matrix4d A, const Coord3d &B) {
+  Coord3d C;
   for (int i = 0; i < 3; i++) {
     C[i] = A[i][3];
     for (int k = 0; k < 3; k++) C[i] += A[i][k] * B[k];
@@ -1301,14 +1295,16 @@ void useMatrix(Matrix4d A, const double B[3], double C[3]) {
   double h = A[3][3];
   for (int k = 0; k < 3; k++) h += A[3][k];
   for (int k = 0; k < 3; k++) C[k] /= h;
+  return C;
 }
 
 /* C <- A(B) */
-void useMatrix(Matrix3d A, const double B[3], double C[3]) {
+Coord3d useMatrix(Matrix3d A, const Coord3d &B) {
+  Coord3d C;
   for (int i = 0; i < 3; i++) {
-    C[i] = 0.;
     for (int k = 0; k < 3; k++) C[i] += A[i][k] * B[k];
   }
+  return C;
 }
 
 /* C <- A */
@@ -1318,19 +1314,6 @@ void assign(const Matrix4d A, Matrix4d C) {
 }
 
 /* C <- A */
-void assign(const double A[3], double C[3]) {
-  C[0] = A[0];
-  C[1] = A[1];
-  C[2] = A[2];
-}
-
-/* C <- A */
-void assign(const float A[3], float C[3]) {
-  C[0] = A[0];
-  C[1] = A[1];
-  C[2] = A[2];
-}
-
 void identityMatrix(Matrix4d m) {
   for (int i = 0; i < 4; i++)
     for (int j = 0; j < 4; j++) m[i][j] = i == j ? 1.0 : 0.0;
@@ -1363,8 +1346,6 @@ void rotateZ(double v, Matrix4d m) {
   assign(m, morig);
   matrixMult(morig, mr, m);
 }
-void zero(double v[3]) { v[0] = v[1] = v[2] = 0.0; }
-
 int testBboxClip(double x1, double x2, double y1, double y2, double z1, double z2,
                  const Matrix4d model, const Matrix4d proj) {
   // Construct axis-aligned bounding box in window space
