@@ -54,6 +54,7 @@
 #include "weather.h"
 
 #include <zlib.h>
+#include <cstdlib>
 
 /* Object coordinates are shifted by DX/DY relative to map coordinates in the API
  * Thus, a flag, ball, etc. placed at (222,225) will appear at (222+DX,225+DY)
@@ -205,7 +206,17 @@ char *ascm_format(const char *str) {
 }
 
 /******************** _ a.k.a. gettext  ********************/
-SCM_DEFINE(underscore, "_", 1, 0, 0, (SCM str), "Translate string if translation available")
+SCM_DEFINE(underscore, "_", 1, 0, 0, (SCM str),
+           "Mark string visible for translation and return it (see 'translate')")
+#define FUNC_NAME s_underscore
+{
+  SCM_ASSERT(scm_is_string(str), str, SCM_ARG1, FUNC_NAME);
+  return str;
+}
+#undef FUNC_NAME
+
+SCM_DEFINE(translate_string, "translate", 1, 0, 0, (SCM str),
+           "Translate string argument if possible (see '_')")
 #define FUNC_NAME s_underscore
 {
   SCM_ASSERT(scm_is_string(str), str, SCM_ARG1, FUNC_NAME);
@@ -214,6 +225,7 @@ SCM_DEFINE(underscore, "_", 1, 0, 0, (SCM str), "Translate string if translation
   SCM out = scm_from_utf8_string(trans);
   free(text);
   return out;
+  return out;
 }
 #undef FUNC_NAME
 
@@ -433,7 +445,8 @@ SCM_DEFINE(add_goal, "add-goal", 4, 0, 0, (SCM x, SCM y, SCM rotate, SCM nextLev
 /************* sign ************/
 SCM_DEFINE(sign, "sign", 6, 1, 0,
            (SCM text, SCM scale, SCM rotation, SCM duration, SCM x, SCM y, SCM z),
-           "Creates a new sign. duration<0 lasts forever. Returns an 'animated' object")
+           "Creates a new sign. Text is translated if possible. duration<0 lasts forever. "
+           "Returns an 'animated' object")
 #define FUNC_NAME s_sign
 {
   SCM_ASSERT(scm_is_string(text), text, SCM_ARG1, FUNC_NAME);
@@ -447,7 +460,7 @@ SCM_DEFINE(sign, "sign", 6, 1, 0,
         pos[2] = scm_to_double(z);
       else
         pos[2] = Game::current->map->getHeight(pos[0], pos[1]) + 2.0;
-      Sign *s = new Sign(sname, scm_to_double(duration), scm_to_double(scale),
+      Sign *s = new Sign(gettext(sname), scm_to_double(duration), scm_to_double(scale),
                          scm_to_double(rotation), pos);
       free(sname);
       return smobAnimated_make(s);
@@ -1298,6 +1311,22 @@ SCM_DEFINE(set_cell_flag, "set-cell-flag", 6, 0, 0,
 }
 #undef FUNC_NAME
 
+/************** get-cell-flag ************/
+SCM_DEFINE(get_cell_flag, "get-cell-flag", 3, 0, 0, (SCM x, SCM y, SCM flag),
+           "Checks if a cell flag is active")
+#define FUNC_NAME s_get_cell_flag
+{
+  SCM_ASSERT(scm_is_integer(x), x, SCM_ARG1, FUNC_NAME);
+  SCM_ASSERT(scm_is_integer(y), y, SCM_ARG2, FUNC_NAME);
+  SCM_ASSERT(scm_is_integer(flag), flag, SCM_ARG5, FUNC_NAME);
+  Cell &c = Game::current->map->cell(scm_to_int(x), scm_to_int(y));
+  if (c.flags & scm_to_int(flag))
+    return SCM_BOOL_T;
+  else
+    return SCM_BOOL_F;
+}
+#undef FUNC_NAME
+
 /************** set-cell-velocity ************/
 SCM_DEFINE(set_cell_velocity, "set-cell-velocity", 6, 0, 0,
            (SCM x0, SCM y0, SCM x1, SCM y1, SCM vx, SCM vy), "Modifies the velocity of a cell")
