@@ -27,6 +27,7 @@
 #include <SDL2/SDL_endian.h>
 #include <SDL2/SDL_image.h>
 #include <zlib.h>
+#include <cstdlib>
 
 /* VISRADIUS is half-width of square of drawable cells */
 #define VISRADIUS 50
@@ -57,7 +58,7 @@ Cell::Cell() {
 }
 
 /* Returns the normal for a point on the edge of the cell */
-void Cell::getNormal(Coord3d normal, int vertex) const {
+void Cell::getNormal(Coord3d* normal, int vertex) const {
   Coord3d v1;
   Coord3d v2;
 
@@ -96,20 +97,20 @@ void Cell::getNormal(Coord3d normal, int vertex) const {
     break;
   case CENTER:
     getNormal(normal, SOUTH + WEST);
-    getNormal(v1, SOUTH + EAST);
-    add(v1, normal, normal);
-    getNormal(v1, NORTH + WEST);
-    add(v1, normal, normal);
-    getNormal(v1, NORTH + EAST);
-    add(v1, normal, normal);
-    normalize(normal);
+    getNormal(&v1, SOUTH + EAST);
+    *normal = *normal + v1;
+    getNormal(&v1, NORTH + WEST);
+    *normal = *normal + v1;
+    getNormal(&v1, NORTH + EAST);
+    *normal = *normal + v1;
+    *normal = *normal / length(*normal);
     return;
   }
-  crossProduct(v1, v2, normal);
-  normalize(normal);
+  *normal = crossProduct(v1, v2);
+  *normal = *normal / length(*normal);
 }
 /* Works on water heights */
-void Cell::getWaterNormal(Coord3d normal, int vertex) const {
+void Cell::getWaterNormal(Coord3d* normal, int vertex) const {
   Coord3d v1;
   Coord3d v2;
 
@@ -148,21 +149,24 @@ void Cell::getWaterNormal(Coord3d normal, int vertex) const {
     break;
   case CENTER:
     getWaterNormal(normal, SOUTH + WEST);
-    getWaterNormal(v1, SOUTH + EAST);
-    add(v1, normal, normal);
-    getWaterNormal(v1, NORTH + WEST);
-    add(v1, normal, normal);
-    getWaterNormal(v1, NORTH + EAST);
-    add(v1, normal, normal);
-    normalize(normal);
+    getWaterNormal(&v1, SOUTH + EAST);
+    *normal = *normal + v1;
+    getWaterNormal(&v1, NORTH + WEST);
+    *normal = *normal + v1;
+    getWaterNormal(&v1, NORTH + EAST);
+    *normal = *normal + v1;
+    *normal = *normal / length(*normal);
     return;
   }
-  crossProduct(v1, v2, normal);
-  normalize(normal);
+  *normal = crossProduct(v1, v2);
+  *normal = *normal / length(*normal);
 }
 
 /* Gives the height of the cell in a specified (floatingpoint) position */
 Real Cell::getHeight(Real x, Real y) const {
+  if (x < 0. || x > 1. || y < 0. || y > 1.) {
+    warning("Require x,y in [0,1], got %f %f", x, y);
+  }
   Real h1, h2, h3, c;
 
   c = heights[CENTER];
@@ -249,8 +253,8 @@ Map::Map(char* filename) {
 
   isBonus = 0;
 
-  snprintf(mapname, sizeof(mapname), _("Unknown track"));
-  snprintf(author, sizeof(author), _("Unknown author"));
+  memset(mapname, 0, sizeof(mapname));
+  memset(author, 0, sizeof(author));
   gp = gzopen(filename, "rb");
   if (gp) {
     int version;
@@ -659,12 +663,12 @@ void Map::fillChunkVBO(Chunk* chunk) const {
       if (c.flags & CELL_SHADE_FLAT) {
         for (size_t i = 0; i < 15; i++) { fcnormal[i / 3][i % 3] = 0.f; }
       } else {
-        double cnormal[5][3];
-        c.getNormal(&cnormal[0][0], 0);
-        c.getNormal(&cnormal[1][0], 1);
-        c.getNormal(&cnormal[2][0], 2);
-        c.getNormal(&cnormal[3][0], 3);
-        c.getNormal(&cnormal[4][0], 4);
+        Coord3d cnormal[5];
+        c.getNormal(&cnormal[0], 0);
+        c.getNormal(&cnormal[1], 1);
+        c.getNormal(&cnormal[2], 2);
+        c.getNormal(&cnormal[3], 3);
+        c.getNormal(&cnormal[4], 4);
         for (size_t i = 0; i < 15; i++) {
           fcnormal[i / 3][i % 3] = (float)cnormal[i / 3][i % 3];
         }
@@ -737,9 +741,34 @@ void Map::fillChunkVBO(Chunk* chunk) const {
                ne.wallColors[Cell::EAST + Cell::NORTH], 0.5f, 0.5f, 0, nv[0], nv[1],
                ewover ? enormal : wnormal);
 
-      // Render the quad. The heights autocorrect orientations
-      const ushort quadmap[12] = {0, 1, 2, 1, 3, 2, 4, 6, 5, 6, 7, 5};
-      for (uint i = 0; i < 12; i++) { widx[j * 12 + i] = j * 8 + quadmap[i]; }
+      // Render the quads, overlapping triangles if twisted
+      const ushort quadmap[6] = {0, 1, 2, 1, 3, 2};
+      const ushort utriang[6] = {0, 1, 3, 0, 3, 2};
+      const ushort dtriang[6] = {0, 1, 2, 1, 2, 3};
+      const ushort* sel;
+      if (cmp(c.heights[Cell::SOUTH + Cell::WEST], ns.heights[Cell::NORTH + Cell::WEST]) *
+              cmp(c.heights[Cell::SOUTH + Cell::EAST], ns.heights[Cell::NORTH + Cell::EAST]) >=
+          0) {
+        sel = quadmap;
+      } else if (c.heights[Cell::SOUTH + Cell::WEST] < ns.heights[Cell::NORTH + Cell::WEST]) {
+        sel = utriang;
+      } else {
+        sel = dtriang;
+      }
+      for (uint i = 0; i < 6; i++) widx[j * 12 + i] = j * 8 + sel[i];
+
+      const ushort aquadmap[6] = {0, 2, 1, 2, 3, 1};
+      const ushort autriang[6] = {0, 2, 1, 2, 3, 1};
+      const ushort adtriang[6] = {0, 3, 1, 2, 3, 0};
+      if (cmp(c.heights[Cell::WEST + Cell::SOUTH], ne.heights[Cell::EAST + Cell::SOUTH]) +
+          cmp(c.heights[Cell::WEST + Cell::NORTH], ne.heights[Cell::EAST + Cell::NORTH])) {
+        sel = aquadmap;
+      } else if (c.heights[Cell::WEST + Cell::SOUTH] > ne.heights[Cell::EAST + Cell::SOUTH]) {
+        sel = autriang;
+      } else {
+        sel = adtriang;
+      }
+      for (uint i = 0; i < 6; i++) widx[j * 12 + 6 + i] = j * 8 + 4 + sel[i];
 
       // Line data ! (thankfully they're all black)
       int s = j * 12;
@@ -770,12 +799,12 @@ void Map::fillChunkVBO(Chunk* chunk) const {
       // Water
       int wvis = c.isWaterVisible();
       if (wvis) {
-        double onormal[5][3];
-        c.getWaterNormal(&onormal[0][0], 0);
-        c.getWaterNormal(&onormal[1][0], 1);
-        c.getWaterNormal(&onormal[2][0], 2);
-        c.getWaterNormal(&onormal[3][0], 3);
-        c.getWaterNormal(&onormal[4][0], 4);
+        Coord3d onormal[5];
+        c.getWaterNormal(&onormal[0], 0);
+        c.getWaterNormal(&onormal[1], 1);
+        c.getWaterNormal(&onormal[2], 2);
+        c.getWaterNormal(&onormal[3], 3);
+        c.getWaterNormal(&onormal[4], 4);
         float fonormal[5][3];
         for (size_t i = 0; i < 15; i++) {
           fonormal[i / 3][i % 3] = (float)onormal[i / 3][i % 3];
@@ -910,20 +939,25 @@ void Map::fillChunkVBO(Chunk* chunk) const {
 void Map::markCellsUpdated(int x1, int y1, int x2, int y2, int changed_walls) {
   /* If the walls of a cell have changed, update its neighbors */
   int dd = changed_walls ? 1 : 0;
-  for (int x = std::min(x1, x2) - dd; x <= std::max(x1, x2) + dd; x++) {
-    for (int y = std::min(y1, y2) - dd; y <= std::max(y1, y2) + dd; y++) {
-      Cell& c = cell(x, y);
-      if (0 <= x && x < width && 0 <= y && y <= width) {
-        Chunk& z = *chunk(x - x % CHUNKSIZE, y - y % CHUNKSIZE);
-        z.is_updated = 1;
-        c.displayListDirty = 1;
 
-        // Extend bounding box
-        for (int k = 0; k < 5; k++) {
-          z.minHeight = std::min(z.minHeight, c.heights[k]);
-          z.minHeight = std::min(z.minHeight, c.waterHeights[k]);
-          z.maxHeight = std::max(z.maxHeight, c.heights[k]);
-          z.maxHeight = std::max(z.maxHeight, c.waterHeights[k]);
+  /* Apply the same clamping to bounds as for cell lookups */
+  int xmin = std::max(std::min(std::min(x1, x2) - dd, width - 1), 0);
+  int xmax = std::max(std::min(std::max(x1, x2) + dd, width - 1), 0);
+  int ymin = std::max(std::min(std::min(y1, y2) - dd, height - 1), 0);
+  int ymax = std::max(std::min(std::max(y1, y2) + dd, height - 1), 0);
+
+  for (int xp = xmin - xmin % CHUNKSIZE; xp <= xmax - xmax % CHUNKSIZE; xp += CHUNKSIZE) {
+    for (int yp = ymin - ymin % CHUNKSIZE; yp <= ymax - ymax % CHUNKSIZE; yp += CHUNKSIZE) {
+      Chunk* z = chunk(xp, yp);
+      z->is_updated = 1;
+      for (int x = std::max(xmin, xp); x <= std::min(xmax, xp + CHUNKSIZE - 1); x++) {
+        for (int y = std::max(ymin, yp); y <= std::min(ymax, yp + CHUNKSIZE - 1); y++) {
+          Cell& c = cells[x + width * y];
+          c.displayListDirty = 1;
+          for (int k = 0; k < 5; k++) {
+            z->minHeight = std::min(z->minHeight, std::min(c.heights[k], c.waterHeights[k]));
+            z->maxHeight = std::max(z->maxHeight, std::max(c.heights[k], c.waterHeights[k]));
+          }
         }
       }
     }
@@ -1020,43 +1054,43 @@ void Map::drawLoop(int x1, int y1, int x2, int y2, int kind) {
       yh = std::max(y1, y2);
   for (int x = xl; x <= xh; x++) {
     Cell& c = cell(x, yl);
-    Coord3d lx = {(double)x, (double)yl, c.heights[Cell::SOUTH + Cell::WEST]};
-    Coord3d ly = {(double)x + 1, (double)yl, c.heights[Cell::SOUTH + Cell::EAST]};
-    assign(lx, rp[k]);
+    Coord3d lx((double)x, (double)yl, c.heights[Cell::SOUTH + Cell::WEST]);
+    Coord3d ly((double)x + 1, (double)yl, c.heights[Cell::SOUTH + Cell::EAST]);
+    rp[k] = lx;
     k++;
-    assign(ly, rp[k]);
+    rp[k] = ly;
     k++;
   }
   for (int y = yl; y <= yh; y++) {
     Cell& c = cell(xh, y);
-    Coord3d lx = {(double)xh + 1, (double)y, c.heights[Cell::SOUTH + Cell::EAST]};
-    Coord3d ly = {(double)xh + 1, (double)y + 1, c.heights[Cell::NORTH + Cell::EAST]};
-    assign(lx, rp[k]);
+    Coord3d lx((double)xh + 1, (double)y, c.heights[Cell::SOUTH + Cell::EAST]);
+    Coord3d ly((double)xh + 1, (double)y + 1, c.heights[Cell::NORTH + Cell::EAST]);
+    rp[k] = lx;
     k++;
-    assign(ly, rp[k]);
+    rp[k] = ly;
     k++;
   }
   for (int x = xh; x >= xl; x--) {
     Cell& c = cell(x, yh);
-    Coord3d lx = {(double)x + 1, (double)yh + 1, c.heights[Cell::NORTH + Cell::EAST]};
-    Coord3d ly = {(double)x, (double)yh + 1, c.heights[Cell::NORTH + Cell::WEST]};
-    assign(lx, rp[k]);
+    Coord3d lx((double)x + 1, (double)yh + 1, c.heights[Cell::NORTH + Cell::EAST]);
+    Coord3d ly((double)x, (double)yh + 1, c.heights[Cell::NORTH + Cell::WEST]);
+    rp[k] = lx;
     k++;
-    assign(ly, rp[k]);
+    rp[k] = ly;
     k++;
   }
   for (int y = yh; y >= yl; y--) {
     Cell& c = cell(xl, y);
-    Coord3d lx = {(double)xl, (double)y + 1, c.heights[Cell::NORTH + Cell::WEST]};
-    Coord3d ly = {(double)xl, (double)y, c.heights[Cell::NORTH + Cell::EAST]};
-    assign(lx, rp[k]);
+    Coord3d lx((double)xl, (double)y + 1, c.heights[Cell::NORTH + Cell::WEST]);
+    Coord3d ly((double)xl, (double)y, c.heights[Cell::NORTH + Cell::EAST]);
+    rp[k] = lx;
     k++;
-    assign(ly, rp[k]);
+    rp[k] = ly;
     k++;
   }
   /* Close the loop */
-  assign(ringPoints[k - 1], ringPoints[0]);
-  assign(ringPoints[1], ringPoints[k]);
+  ringPoints[0] = ringPoints[k - 1];
+  ringPoints[k] = ringPoints[1];
 
   GLfloat* data = new GLfloat[8 * 2 * npts];
   GLfloat width = 0.05f;
@@ -1064,30 +1098,23 @@ void Map::drawLoop(int x1, int y1, int x2, int y2, int kind) {
   char* pos = (char*)data;
   int nontrivial = 0;
   for (int i = 1; i < npts + 1; i++) {
-    Coord3d prev, cur, nxt;
-    assign(ringPoints[i - 1], prev);
-    assign(ringPoints[i], cur);
-    assign(ringPoints[i + 1], nxt);
-    Coord3d dir1, dir2, dir;
-    sub(nxt, cur, dir1);
-    sub(cur, prev, dir2);
+    Coord3d prev = ringPoints[i - 1], cur = ringPoints[i], nxt = ringPoints[i + 1];
+    Coord3d dir1 = nxt - cur, dir2 = cur - prev;
     if (length(dir1) < 1e-2 && length(dir1) < 1e-2) { continue; }
-    if (length(dir1) < 1e-2) { assign(dir2, dir1); }
-    if (length(dir2) < 1e-2) { assign(dir1, dir2); }
-    normalize(dir1);
-    normalize(dir2);
-    add(dir1, dir2, dir);
+    if (length(dir1) < 1e-2) { dir1 = dir2; }
+    if (length(dir2) < 1e-2) { dir2 = dir1; }
+    dir1 = dir1 / length(dir1);
+    dir2 = dir2 / length(dir2);
+    Coord3d dir = dir1 + dir2;
     if (length(dir) < 1e-2) { continue; }
     nontrivial++;
-    normalize(dir);
+    dir = dir / length(dir);
     dir[2] = 0;
-    Coord3d up = {0., 0., 1.};
-    Coord3d res;
-    crossProduct(up, dir, res);
+    Coord3d up(0., 0., 1.);
+    Coord3d res = crossProduct(up, dir);
     res[2] = 0.;
-    if (length(res) > 0) normalize(res);
-    Coord3d anti;
-    crossProduct(dir, res, anti);
+    if (length(res) > 0) res = res / length(res);
+    Coord3d anti = crossProduct(dir, res);
     GLfloat off = 1e-2;
     kind = nontrivial % 2;
     pos += packObjectVertex(pos, cur[0] + width * res[0] + off * anti[0],
@@ -1243,12 +1270,12 @@ int Map::save(char* pathname, int x, int y) {
 
   gzFile gp = gzopen(pathname, "wb9");
   if (!gp) return 0;
-  startPosition[0] = x;
-  startPosition[1] = y;
-  startPosition[2] = cell(x, y).heights[Cell::CENTER];
+  Coord3d tmp = startPosition;
+  startPosition = Coord3d(x, y, cell(x, y).heights[Cell::CENTER]);
   int32_t data[6];
   for (int i = 0; i < 3; i++)
     data[i] = saveInt((int32_t)startPosition[i]);  // no decimal part needed
+  tmp = startPosition;
   data[3] = saveInt((int32_t)width);
   data[4] = saveInt((int32_t)height);
   data[5] = saveInt((int32_t)version);
