@@ -32,6 +32,57 @@
 #include <SDL2/SDL_keyboard.h>
 #include <SDL2/SDL_mouse.h>
 
+Uint32 getFilteredRelativeMouse(double *xrate, double *yrate) {
+  /* Finite impulse response filter to smooth mouse input */
+  const int N = 1000;
+  const double T = 0.2;
+  static int fir[N][2];
+  static double ts[N];
+  static int first = true;
+  if (first) {
+    for (int i = 0; i < N; i++) {
+      fir[i][0] = 0;
+      fir[i][1] = 0;
+      ts[i] = -1. - T;
+    }
+    first = false;
+  }
+
+  double now = Game::current->gameTime;
+
+  int mouseX, mouseY;
+  Uint32 mouseState = SDL_GetRelativeMouseState(&mouseX, &mouseY);
+  if (mouseX || mouseY) {
+    /* Add value to filter */
+
+    int cx = mouseX, cy = mouseY;
+    double ct = now;
+    int i = 0;
+    for (; ts[i] > now - T; i++) {
+      std::swap(fir[i][0], cx);
+      std::swap(fir[i][1], cy);
+      std::swap(ts[i], ct);
+    }
+    std::swap(fir[i][0], cx);
+    std::swap(fir[i][1], cy);
+    std::swap(ts[i], ct);
+  }
+
+  int sx = 0, sy = 0;
+  for (int i = 0; ts[i] > now - T; i++) {
+    if (ts[i] < now - T / 2) {
+      sx += fir[i][0];
+      sy += fir[i][1];
+    } else {
+      sx += 2 * fir[i][0];
+      sy += 2 * fir[i][1];
+    }
+  }
+  *xrate = sx / T / 2;
+  *yrate = sy / T / 2;
+  return mouseState;
+}
+
 Player::Player() : Ball() {
   inTheAir = 0;
   inPipe = 0;
@@ -73,9 +124,7 @@ void Player::draw() {
   Ball::draw();
 }
 void Player::tick(Real t) {
-  double dx, dy;
-  int superAccelerate = 0;
-  static time_t lastTick = 0;
+  static Real timeFraction = 0.;
 
   /* Never let us drop below 0 points, it just looks silly */
   if (score < 0) score = 0;
@@ -83,18 +132,18 @@ void Player::tick(Real t) {
   if (!Game::current) return;
   if (!playing) return;
 
-  dx = 0.0;
-  dy = 0.0;
   Map *map = Game::current->map;
   health += t * 0.4;
   if (health > 1.0) health = 1.0;
 
-  time_t thisTick = time(NULL);
-  if (thisTick != lastTick && !hasWon) {
-    timeLeft--;
-    if (timeLeft < 15) playEffect(SFX_TIME_WARNING);
+  timeFraction += t;
+  if (timeFraction > 1.0) {
+    timeFraction -= 1.0;
+    if (!hasWon) {
+      timeLeft--;
+      if (timeLeft < 15) playEffect(SFX_TIME_WARNING);
+    }
   }
-  lastTick = thisTick;
   if (timeLeft < 1) {
     /* Only die from running out of time if we are not running
        in sandbox mode. */
@@ -123,10 +172,10 @@ void Player::tick(Real t) {
   } else
     oxygen = std::min(1.0, oxygen + (t / 4.0) / Game::current->oxygenFactor);
 
+  double dx = 0., dy = 0.;
   /* Joysticks */
   if (Settings::settings->hasJoystick()) {
     if (Settings::settings->joystickButton(0)) key(' ');
-    if (Settings::settings->joystickButton(1)) superAccelerate = 1;
 
     double joyX = Settings::settings->joystickX();
     double joyY = Settings::settings->joystickY();
@@ -134,49 +183,50 @@ void Player::tick(Real t) {
     if (joyX < 0.1 && joyX > -0.1) joyX = 0.0;
     if (joyY < 0.1 && joyY > -0.1) joyY = 0.0;
 
-    dx = ((double)joyX) * Settings::settings->mouseSensitivity * t * 200.0;
-    dy = ((double)joyY) * Settings::settings->mouseSensitivity * t * 200.0;
+    dx = ((double)joyX) * Settings::settings->mouseSensitivity;
+    dy = ((double)joyY) * Settings::settings->mouseSensitivity;
   }
 
   /* Give only *relative* mouse movements */
   if (!Settings::settings->ignoreMouse && !(SDL_GetModState() & KMOD_CAPS)) {
-    int mouseX, mouseY;
-
-    Uint32 mouseState = SDL_GetRelativeMouseState(&mouseX, &mouseY);
-
-    if (mouseState & SDL_BUTTON_RMASK) superAccelerate = 1;
-    if ((mouseX || mouseY)) {
-      dx = mouseX * Settings::settings->mouseSensitivity * 0.1;
-      dy = mouseY * Settings::settings->mouseSensitivity * 0.1;
+    double sx, sy;
+    getFilteredRelativeMouse(&sx, &sy);
+    if (sx || sy) {
+      dx = sx * Settings::settings->mouseSensitivity * 0.0005;
+      dy = sy * Settings::settings->mouseSensitivity * 0.0005;
     }
   }
 
   /* Handle keyboard steering */
   const Uint8 *keystate = SDL_GetKeyboardState(NULL);
   int shift = SDL_GetModState() & (KMOD_LSHIFT | KMOD_RSHIFT);
-  double kscale = shift ? 200.0 : 100.0;
 
-  if (keystate[SDL_SCANCODE_UP] || keystate[SDL_SCANCODE_KP_8]) { dy = -kscale * t; }
-  if (keystate[SDL_SCANCODE_DOWN] || keystate[SDL_SCANCODE_KP_2]) { dy = kscale * t; }
-  if (keystate[SDL_SCANCODE_LEFT] || keystate[SDL_SCANCODE_KP_4]) { dx = -kscale * t; }
-  if (keystate[SDL_SCANCODE_RIGHT] || keystate[SDL_SCANCODE_KP_6]) { dx = kscale * t; }
+  int ix = 0, iy = 0;
+  if (keystate[SDL_SCANCODE_UP] || keystate[SDL_SCANCODE_KP_8]) iy--;
+  if (keystate[SDL_SCANCODE_DOWN] || keystate[SDL_SCANCODE_KP_2]) iy++;
+  if (keystate[SDL_SCANCODE_LEFT] || keystate[SDL_SCANCODE_KP_4]) ix--;
+  if (keystate[SDL_SCANCODE_RIGHT] || keystate[SDL_SCANCODE_KP_6]) ix++;
   if (keystate[SDL_SCANCODE_KP_7] || keystate[SDL_SCANCODE_Q]) {
-    dx = -kscale * t;
-    dy = -kscale * t;
+    ix--;
+    iy--;
   }
   if (keystate[SDL_SCANCODE_KP_9] || keystate[SDL_SCANCODE_W]) {
-    dx = kscale * t;
-    dy = -kscale * t;
+    ix++;
+    iy--;
   }
   if (keystate[SDL_SCANCODE_KP_1] || keystate[SDL_SCANCODE_A]) {
-    dx = -kscale * t;
-    dy = kscale * t;
+    ix--;
+    iy++;
   }
   if (keystate[SDL_SCANCODE_KP_3] || keystate[SDL_SCANCODE_S]) {
-    dx = kscale * t;
-    dy = kscale * t;
+    ix++;
+    iy++;
+  }
+  if (ix || iy) {
+    double len = std::sqrt(ix * ix + iy * iy);
+    dx = ix / len;
+    dy = iy / len;
   }
-  if (keystate[SDL_SCANCODE_KP_ENTER] || keystate[SDL_SCANCODE_RETURN]) superAccelerate = 1;
 
   /* rotate control as by settings->rotateArrows */
   {
@@ -194,69 +244,32 @@ void Player::tick(Real t) {
     dx = tmp;
   }
 
-  /* Cap is the maximum normal acceleration we are allowed to perform (on average) per second
-   */
-  double cap = 140.0 - 10.0 * Settings::settings->difficulty;
-  if (modTimeLeft[MOD_SPEED]) {
-    dx *= 1.5;
-    dy *= 1.5;
-    cap = 200.0;
-  }
-  if (modTimeLeft[MOD_DIZZY]) cap = cap / 2.0;
-
-  /* Limit size of actual movement according to "cap" */
-  /* Uses a trick to avoid capping away movements from integer only devices (eg. mouse) while
-   * having a high FPS */
-  moveBurst = std::min(cap * 0.2, moveBurst + cap * t);
-  double len = sqrt(dx * dx + dy * dy);
-  if (len > moveBurst) {
-    dx = dx * moveBurst / len;
-    dy = dy * moveBurst / len;
-    len = moveBurst;
-  }
-  moveBurst -= len;
-
-  /*
-  if(modTimeLeft[MOD_DIZZY]) {
-        double tmp = dx * sin(Game::current->gameTime*M_PI*0.1) + dy *
-  cos(Game::current->gameTime*M_PI*0.1);
-        dy = dx * cos(Game::current->gameTime*M_PI*0.1) - dy *
-  sin(Game::current->gameTime*M_PI*0.1);
-        dx = tmp;
-        }*/
-
-  /* Do the movement. Also, if nitro is active then create debris after the ball */
-
-  if (!modTimeLeft[MOD_FROZEN] && !hasWon && is_on) {
-    rotation[0] -= 0.025 * (dy - dx);
-    rotation[1] -= 0.025 * (dx + dy);
-
-    if (superAccelerate && modTimeLeft[MOD_NITRO]) {
-      double dirX = rotation[0];
-      double dirY = rotation[1];
-      double len = sqrt(dirX * dirX + dirY * dirY);
-      static double nitroDebris = 0.0;
-      nitroDebris += t;
-      while (nitroDebris > 0.1) {
-        nitroDebris -= 0.1;
-        Debris *d = new Debris(this, position, velocity, 1.0 + frandom() * 2.0);
-        d->position[0] += (frandom() - 0.5) * radius - velocity[0] * 0.1;
-        d->position[1] += (frandom() - 0.5) * radius - velocity[1] * 0.1;
-        d->position[2] += (frandom() - 0.5) * radius;
-        d->modTimeLeft[MOD_GLASS] = -1.0;
-        d->primaryColor[0] = 0.1;
-        d->primaryColor[1] = 0.6;
-        d->primaryColor[2] = 0.1;
-      }
-      if (len > 0.0) {
-        dirX /= len;
-        dirY /= len;
-        rotation[0] += dirX * t * 10.0;
-        rotation[1] += dirY * t * 10.0;
-      }
-    }
+  /* Cap dx/dy to have total radius 1. */
+  double len = std::sqrt(dx * dx + dy * dy);
+  if (len > 1.) {
+    dx /= len;
+    dy /= len;
   }
 
+  // Rotate by 45 degrees, introduce sqrt2 scale factor
+  double tx = -(dy - dx);
+  double ty = -(dy + dx);
+  dx = tx;
+  dy = ty;
+
+  if (shift) {
+    dx *= 0.5;
+    dy *= 0.5;
+  }
+
+  /* set inherent base acceleration*/
+  acceleration = 3.5 - 0.25 * Settings::settings->difficulty;
+
+  if (!hasWon && is_on) {
+    Ball::drive(dx, dy);
+  } else {
+    Ball::drive(0., 0.);
+  }
   if (!hasWon)
     friction = 1.0;
   else
@@ -274,23 +287,12 @@ void Player::key(int c) {
   }
 }
 void Player::jump() {
-  if (!inTheAir) {
-    if (Game::current->map->cell((int)position[0], (int)position[1]).flags & CELL_ACID)
-      velocity[2] +=
-          1.0 * Game::current->jumpFactor * (1.2 - 0.1 * Settings::settings->difficulty);
-    else if (modTimeLeft[MOD_JUMP])
-      velocity[2] +=
-          5.0 * Game::current->jumpFactor * (1.2 - 0.1 * Settings::settings->difficulty);
-    else
-      velocity[2] +=
-          2.0 * Game::current->jumpFactor * (1.2 - 0.1 * Settings::settings->difficulty);
-    double dh = position[2] - Game::current->map->getHeight(position[0], position[1]);
-    if (dh > 0.20)
-      dh = 0.20;  // we are not supposed to be not "inTheAir" and also have dh > 0.20!
-    position[2] += std::min(0.2, 0.20 - dh);
-    inTheAir = true;
-  }
+  double jumpStrength =
+      Game::current->jumpFactor * (1.2 - 0.1 * Settings::settings->difficulty);
+  jumpStrength *= modTimeLeft[MOD_JUMP] ? 5.0 : 3.0;
+  Ball::queueJump(jumpStrength);
 }
+
 void Player::die(int how) {
   Map *map = Game::current->map;
   /* immortal when not playing (i.e., at level finish) */
@@ -318,10 +320,8 @@ void Player::die(int how) {
   Coord3d pos, vel;
   if (how == DIE_ACID) {
     GLfloat acidColor[4] = {0.1, 0.8, 0.1, 0.5};
-    Coord3d vel = {0.0, 0.0, 0.0};
-    Coord3d center;
-    assign(position, center);
-    center[2] = map->getHeight(center[0], center[1]);
+    Coord3d vel;
+    Coord3d center(position[0], position[1], map->getHeight(position[0], position[1]));
     new Splash(center, vel, acidColor, 32.0, radius);
   } else
     for (int i = 0; i < 4; i++)
@@ -354,7 +354,7 @@ void Player::die(int how) {
 void Player::setStartVariables() {
   sink = 0.0;
   rotation[0] = rotation[1] = 0.;
-  zero(velocity);
+  velocity = Coord3d();
   playing = true;
   alive = 1;
   hasWon = 0;
@@ -367,7 +367,7 @@ void Player::setStartVariables() {
 
 void Player::restart(Coord3d pos) {
   setStartVariables();
-  assign(pos, position);
+  position = pos;
   position[2] = Game::current->map->getHeight(position[0], position[1]) + radius;
   modTimeLeft[MOD_DIZZY] = 0.0;
 
@@ -381,7 +381,7 @@ void Player::setHealth(Real d) {
   if (d < health) health = d;
   if (health < 0.0) health = 0.0;
 }
-Boolean Player::crash(Real speed) {
+bool Player::crash(Real speed) {
   double espeed =
       modTimeLeft[MOD_GLASS] ? (1.5 * speed) / crashTolerance : speed / crashTolerance;
   setHealth(1.0 - espeed);
