@@ -30,6 +30,9 @@
 #include <locale.h>
 #include <string.h>
 #include <sys/stat.h>
+#include <cstdlib>
+
+extern double timeDilationFactor;
 
 /* These are the names/codes codes of languages available from the settings
    menu. The name of languages should appear in the respective native
@@ -77,6 +80,8 @@ Settings::Settings() {
   nLevelSets = 0;
   joy_center[0] = 0;
   joy_center[1] = 0;
+  shadowTexsize = 9;
+  timeCompression = 0;
   memset(specialLevel, 0, sizeof(specialLevel));
   memset(levelSets, 0, sizeof(levelSets));
 
@@ -100,18 +105,21 @@ Settings::Settings() {
         warning("Configuration file should be a series of (key value) tuples.");
         continue;
       }
-      const int intnum = 18, realnum = 3;
+      const int intnum = 21, realnum = 3;
       const char *intkeys[intnum] = {
-          "gfx-details",    "show-fps",     "is-windowed",     "resolution", "color-depth",
-          "joystick-index", "joy_center-x", "joy_center-y",    "joy-left",   "joy-right",
-          "joy-up",         "joy-down",     "rotate-steering", "language",   "ignore-mouse",
-          "do-reflections", "do-shadows",   "vsync-on"};
+          "gfx-details",      "show-fps",       "is-windowed",    "resolution",
+          "color-depth",      "joystick-index", "joy_center-x",   "joy_center-y",
+          "joy-left",         "joy-right",      "joy-up",         "joy-down",
+          "rotate-steering",  "language",       "ignore-mouse",   "do-reflections",
+          "do-shadows",       "vsync-on",       "shadow-texsize", "time-compression",
+          "sandbox-available"};
 
-      int *intdests[intnum] = {&gfx_details,    &showFPS,       &is_windowed,   &resolution,
-                               &colorDepth,     &joystickIndex, &joy_center[0], &joy_center[1],
-                               &joy_left,       &joy_right,     &joy_up,        &joy_down,
-                               &rotateSteering, &language,      &ignoreMouse,   &doReflections,
-                               &doShadows,      &vsynced};
+      int *intdests[intnum] = {
+          &gfx_details,     &showFPS,       &is_windowed,    &resolution,    &colorDepth,
+          &joystickIndex,   &joy_center[0], &joy_center[1],  &joy_left,      &joy_right,
+          &joy_up,          &joy_down,      &rotateSteering, &language,      &ignoreMouse,
+          &doReflections,   &doShadows,     &vsynced,        &shadowTexsize, &timeCompression,
+          &sandboxAvailable};
       const char *realkeys[realnum] = {"sfx-volume", "music-volume", "mouse-sensitivity"};
       double *realdests[realnum] = {&sfxVolume, &musicVolume, &mouseSensitivity};
 
@@ -144,6 +152,8 @@ Settings::Settings() {
     }
     scm_close(ip);
   }
+  /* apply time compression immediately */
+  timeDilationFactor = std::pow(2.0, timeCompression / 6.0);
 }
 void Settings::loadLevelSets() {
   /* Load all levelsets */
@@ -225,13 +235,11 @@ void Settings::loadLevelSet(const char *setname, const char *shortname) {
       warning(reqmnt, setname);
       continue;
     }
-    char *sval = NULL;
     char *eval = NULL;
     if (scm_is_string(val)) {
       eval = scm_to_utf8_string(val);
-      sval = eval;
-      int len = strlen(sval);
-      if (len > 0 && sval[len - 1] == '\n') { sval[len - 1] = 0; }
+      int len = strlen(eval);
+      if (len > 0 && eval[len - 1] == '\n') { eval[len - 1] = 0; }
     } else {
       if (scm_to_int(scm_length(val)) != 2 || SCM_CAR(val) != scm_from_utf8_symbol("_") ||
           !scm_is_string(SCM_CADR(val))) {
@@ -239,33 +247,32 @@ void Settings::loadLevelSet(const char *setname, const char *shortname) {
         continue;
       }
       eval = scm_to_utf8_string(SCM_CADR(val));
-      sval = gettext(eval);
     }
 
     char *skey = scm_to_utf8_string(scm_symbol_to_string(key));
     if (!strcmp(skey, "name")) {
-      strncpy(levelSets[nLevelSets].name, sval, 256);
+      strncpy(levelSets[nLevelSets].name, eval, 256);
       levelSets[nLevelSets].name[255] = '\0';
     } else if (!strcmp(skey, "first-level-file")) {
-      strncpy(levelSets[nLevelSets].startLevel, sval, 256);
+      strncpy(levelSets[nLevelSets].startLevel, eval, 256);
       levelSets[nLevelSets].startLevel[255] = '\0';
     } else if (!strcmp(skey, "first-level-name")) {
-      strncpy(levelSets[nLevelSets].startLevelName, sval, 256);
+      strncpy(levelSets[nLevelSets].startLevelName, eval, 256);
       levelSets[nLevelSets].startLevelName[255] = '\0';
     } else if (!strcmp(skey, "desc1")) {
-      strncpy(&levelSets[nLevelSets].description[0][0], sval, 120);
+      strncpy(&levelSets[nLevelSets].description[0][0], eval, 120);
       levelSets[nLevelSets].description[0][119] = '\0';
     } else if (!strcmp(skey, "desc2")) {
-      strncpy(&levelSets[nLevelSets].description[1][0], sval, 120);
+      strncpy(&levelSets[nLevelSets].description[1][0], eval, 120);
       levelSets[nLevelSets].description[1][119] = '\0';
     } else if (!strcmp(skey, "desc3")) {
-      strncpy(&levelSets[nLevelSets].description[2][0], sval, 120);
+      strncpy(&levelSets[nLevelSets].description[2][0], eval, 120);
       levelSets[nLevelSets].description[2][119] = '\0';
     } else if (!strcmp(skey, "desc4")) {
-      strncpy(&levelSets[nLevelSets].description[3][0], sval, 120);
+      strncpy(&levelSets[nLevelSets].description[3][0], eval, 120);
       levelSets[nLevelSets].description[3][119] = '\0';
     } else if (!strcmp(skey, "desc5")) {
-      strncpy(&levelSets[nLevelSets].description[4][0], sval, 120);
+      strncpy(&levelSets[nLevelSets].description[4][0], eval, 120);
       levelSets[nLevelSets].description[4][119] = '\0';
     } else {
       warning("Unidentified key '%s' in level set file %s", skey, setname);
@@ -332,8 +339,11 @@ void Settings::save() {
     fprintf(fp, "(ignore-mouse %d)\n", ignoreMouse);
     fprintf(fp, "(do-reflections %d)\n", doReflections);
     fprintf(fp, "(do-shadows %d)\n", doShadows);
+    fprintf(fp, "(shadow-texsize %d)\n", shadowTexsize);
     fprintf(fp, "(language %d)\n", language);
     fprintf(fp, "(vsync-on %d)\n", vsynced);
+    fprintf(fp, "(time-compression %d)\n", timeCompression);
+    fprintf(fp, "(sandbox-available %d)\n", sandboxAvailable);
     fclose(fp);
   }
 }
