@@ -58,6 +58,7 @@ class SettingsMode : public GameMode {
     MENU_GFX_DETAILS,
     MENU_DO_REFLECTIONS,
     MENU_DO_SHADOWS,
+    MENU_SHADOW_TEXSIZE,
 
     MENU_USEMOUSE,
     MENU_SENSITIVITY,
@@ -67,6 +68,8 @@ class SettingsMode : public GameMode {
     MENU_MUSIC_VOLUME,
     MENU_SFX_VOLUME,
     MENU_LANGUAGE,
+    MENU_SANDBOX_AVAILABLE,
+    MENU_TIME_COMPRESSION,
 
     MENU_RESOLUTION_OK,
     MENU_RESOLUTION_BAD,
