@@ -51,7 +51,7 @@ class SetupMode : public GameMode {
 
   GLuint screenshot;
   GLfloat scrshtCoord[4];
-  GLuint scrshtW, scrshtH;
+  GLint scrshtW, scrshtH;
 };
 
 #endif
