@@ -30,7 +30,7 @@
 #include "spike.h"
 
 SideSpike::SideSpike(Coord3d position, Real speed, Real phase, int side) {
-  assign(position, this->position);
+  this->position = position;
   this->position[2] = Game::current->map->getHeight(position[0], position[1]) + 0.0;
   this->speed = speed;
   while (phase < 0.0) phase += 1.0;
@@ -60,14 +60,14 @@ SideSpike::SideSpike(Coord3d position, Real speed, Real phase, int side) {
 int SideSpike::generateBuffers(GLuint *&idxbufs, GLuint *&databufs) {
   allocateBuffers(1, idxbufs, databufs);
 
-  int nfacets = 6;
+  const int nfacets = 6;
   GLfloat data[(4 * nfacets) * 8];
   ushort idxs[3 * nfacets][3];
 
   Matrix4d frommtx;
   identityMatrix(frommtx);
 
-  Coord3d pos = {position[0], position[1], position[2] + 0.25};
+  Coord3d pos(position[0], position[1], position[2] + 0.25);
   switch (side) {
   case 1:
     rotateY(M_PI / 2, frommtx);
